import dateutil.parser
from django.contrib.auth.models import User
from status.models import Thread, Message, DailyLog
from pytz import timezone
from django.core.exceptions import ObjectDoesNotExist


def log(data, members, thread_id):
    thread = Thread.objects.get(id=thread_id)

    for entry in data:
        try:
            user = User.objects.get(email=entry['email'])
            timestamp = dateutil.parser.parse(entry['timestamp'])
            logDate = dateutil.parser.parse(entry['date']).date()

            msgObj, msgCreated = Message.objects.get_or_create(
                member=user,
                date=logDate,
                timestamp=timestamp,
                thread=thread,
                message=entry['message']
            )
            msgObj.save()
            if msgCreated:
                obj, created = DailyLog.objects.get_or_create(
                    date=logDate,
                    thread=thread,
                )
                if created:
                    obj.members.add(*members)
                    obj.didNotSend.add(*members)

                if user in obj.members.all():
                    dueTime = timezone("Asia/Calcutta").localize(
                        dateutil.parser.parse(entry['date']).replace(hour=int(thread.dueTime[:2]),
                                                                     minute=int(thread.dueTime[2:])))

                    if user in obj.didNotSend.all():
                        if timestamp > dueTime:
                            obj.late.add(user)
                        obj.didNotSend.remove(user)
                obj.save()

        except ObjectDoesNotExist:
            pass



import httplib2
import base64
import pytz

from oauth2client import file, client, tools
from apiclient import discovery
from email.utils import parsedate_to_datetime


class GMailFetcher(object):
    def __init__(self, subject, d):
        self.service = self.startService()
        self.subject = subject
        self.date = d

        threads = self.getThreads()
        if threads is not None:
            messages = self.getThreadMessages(threads)
            if messages is not None:
                self.logs = self.processMessages(messages)

    @staticmethod
    def startService():
        store = file.Storage('credentials.json')
        credentials = store.get()
        scope = 'https://www.googleapis.com/auth/gmail.readonly'
        if not credentials or credentials.invalid:
            flow = client.flow_from_clientsecrets('client_secret.json', scope)
            credentials = tools.run_flow(flow, store)
        http = credentials.authorize(httplib2.Http())
        return discovery.build('gmail', 'v1', http=http)

    def getThreads(self):
        response = self.service.users().messages().list(
            userId='me',
            q=self.subject
        ).execute()
        threads = []
        if 'messages' in response:
            for msg in response['messages']:
                if msg['threadId'] not in threads:
                    threads.append(msg['threadId'])
            return threads
        return None

    def getThreadMessages(self, threads):
        msgs = []
        for thread_id in threads:
            thread = self.service.users().threads().get(userId='me', id=thread_id, format='full').execute()

            for msg in thread['messages']:
                msgs.append(msg)
        return msgs

    def processMessages(self, msgs):
        log = []
        for msg in msgs:

            header_data = msg["payload"]["headers"]
            for data in header_data:
                if "From" == data["name"]:
                    email_id = data["value"]
                    if '<' in email_id:
                        start = email_id.find('<')
                        end = email_id.find('>')
                        email_id = email_id[start + 1: end]
                if "Received" == data["name"]:
                    timestamp = parsedate_to_datetime(data["value"].split(';', 1)[-1]).astimezone(
                        pytz.timezone("Asia/Calcutta"))

            MsgB64 = msg["payload"]['parts'][0]['body']['data'].replace("-", "+").replace("_", "/")
            Msg = base64.b64decode(bytes(MsgB64, 'UTF-8')).decode('UTF-8')

            Msg = "<br />".join(Msg.split("\r\n"))
            Msg = Msg.split("wrote:<br /><br />>")[0]
            Msg = Msg.rsplit("On ", 1)[0]
            Msg = Msg.split("-- <br />You received this message")[0]

            log.append({
                'email': email_id,
                'date': self.date,
                'timestamp': timestamp.isoformat(),
                'message': Msg
            })

        return log


from django.contrib import admin
from .models import *
from easy_select2 import select2_modelform
from import_export.admin import ImportExportModelAdmin, ExportActionMixin
from import_export import resources


@admin.register(Thread)
class ThreadAdmin(admin.ModelAdmin):
    fieldsets = (
        (None, {
            'fields': ('name', 'isActive', 'enableGroupNotification')
        }),
        ('Timings', {
            'fields': (('days', 'generationTime', 'dueTime', 'logTime'),)
        }),
        ('Mailing', {
            'fields': (('email',), 'threadMessage',)
        }),
        ('Reporting', {
            'fields': ('footerMessage',)
        }),
    )
    list_display = ('name', 'isActive', 'enableGroupNotification', 'generationTime', 'dueTime', 'logTime')
    search_fields = ['name']
    select2 = select2_modelform(Thread, attrs={'width': '250px'})
    form = select2


@admin.register(Message)
class MessageAdmin(admin.ModelAdmin):
    fields = [
        ('member', 'thread'),
        ('date', 'timestamp'),
        'message'
    ]
    list_display = ('member', 'date', 'timestamp', 'thread')
    search_fields = ('member__username', 'thread__name')
    filter = ('thread',)


class DailyLogResource(resources.ModelResource):
    class Meta:
        model = DailyLog


@admin.register(DailyLog)
class DailyLogAdmin(ImportExportModelAdmin, ExportActionMixin, admin.ModelAdmin):
    fields = [
        ('date', 'thread'),
        'members',
        'late',
        'didNotSend'
    ]
    list_display = ('date', 'thread', 'totalCount', 'lateCount', 'didNotSendCount')
    select2 = select2_modelform(DailyLog, attrs={'width': '300px'})
    form = select2
    resource_class = DailyLogResource

    @staticmethod
    def totalCount(obj):
        return obj.members.count()

    @staticmethod
    def lateCount(obj):
        return obj.late.count()

    @staticmethod
    def didNotSendCount(obj):
        return obj.didNotSend.count()



from django.db import models
from django.contrib.auth.models import User
from ckeditor.fields import RichTextField


class Thread(models.Model):
    name = models.CharField(max_length=200,  verbose_name="Name of Thread")
    isActive = models.BooleanField(verbose_name="Is Thread Active", default=True)
    enableGroupNotification = models.BooleanField(verbose_name="Should Send Report to Group?", default=True)
    email = models.EmailField(max_length=250, verbose_name="Thread Email")
    days = models.CharField(max_length=50, null=True, blank=True, verbose_name="Days # to be active, leave blank for all days")
    generationTime = models.CharField(max_length=50, verbose_name="Generation Time")
    dueTime = models.CharField(max_length=50, verbose_name="Due Time")
    logTime = models.CharField(max_length=50, verbose_name="Log Time")
    threadMessage = RichTextField(max_length=2000, verbose_name="Thread Email Message")
    footerMessage = models.CharField(max_length=500, verbose_name="Telegram Footer Message", null=True, blank=True)

    def __str__(self):
        return self.name


class Message(models.Model):
    member = models.ForeignKey(User, on_delete=models.CASCADE)
    timestamp = models.DateTimeField()
    date = models.DateField()
    thread = models.ForeignKey(Thread, on_delete=models.CASCADE)
    message = RichTextField(max_length=2500, verbose_name="Thread Body", null=True, blank=True)

    class Meta:
        verbose_name_plural = "Messages"
        verbose_name = "Message"

    def __str__(self):
        return self.member.username


class DailyLog(models.Model):
    date = models.DateField()
    thread = models.ForeignKey(Thread, on_delete=models.CASCADE)
    members = models.ManyToManyField(User, related_name='threadMembers', blank=True)
    late = models.ManyToManyField(User, related_name='lateStatusLog', blank=True)
    didNotSend = models.ManyToManyField(User, related_name='didNotSendStatusLog', blank=True)

    class Meta:
        verbose_name_plural = "Daily Logs"
        verbose_name = "Daily Log"

    def __str__(self):
        return str(self.date) + ' - ' + self.thread.name
